package api

import (
	"net/http"

	"github.com/gin-gonic/gin"
)

type Error struct {
	StatusCode int    `json:"-"`
	Code       int    `json:"code"`
	Msg        string `json:"msg"`
}

var (
	Success     = NewError(http.StatusOK, 0, "success")
	ServerError = NewError(http.StatusInternalServerError, 200500, "system error!")
	NotFound    = NewError(http.StatusNotFound, 200404, http.StatusText(http.StatusNotFound))
)

func OtherError(message string) *Error {
	return NewError(http.StatusForbidden, 100403, message)
}

func (e *Error) Error() string {
	return e.Msg
}

func NewError(statusCode, Code int, msg string) *Error {
	return &Error{
		StatusCode: statusCode,
		Code:       Code,
		Msg:        msg,
	}
}

// 404
func HandleNotFound(c *gin.Context) {
	err := NotFound
	c.JSON(err.StatusCode, err)
	return
}

func ErrHandler() gin.HandlerFunc {
	return func(c *gin.Context) {
		c.Next()
		if length := len(c.Errors); length > 0 {
			e := c.Errors[length-1]
			err := e.Err
			if err != nil {
				var Err *Error
				if e, ok := err.(*Error); ok {
					Err = e
				} else if e, ok := err.(error); ok {
					Err = OtherError(e.Error())
				} else {
					Err = ServerError
				}
				c.JSON(Err.StatusCode, Err)
				return
			}
		}
	}
}

//func Timeout() gin.HandlerFunc {
//	return func(c *gin.Context) {
//		c.Next()
//		timeout.New(
//			timeout.WithTimeout(100*time.Microsecond),
//			timeout.WithHandler(emptySuccessResponse))
//	}
//}
//
//func emptySuccessResponse(c *gin.Context) {
//	time.Sleep(200 * time.Microsecond)
//	c.String(http.StatusOK, "")
//}
